package com.jiuqi.sql;

import java.io.IOException;
import java.io.Reader;

public abstract class SQLScriptExcutor
{
    private Reader reader;
    private boolean isIgnoreComment;
    private boolean isString;
    private boolean isBlockComment;
    private boolean isLineComment;
    private StringBuffer sql;
    private StringBuffer line;
    
    public SQLScriptExcutor(final Reader sqlReader, final boolean isIgnoreComment) {
        this.reader = sqlReader;
        this.isIgnoreComment = isIgnoreComment;
        this.sql = new StringBuffer();
        this.line = new StringBuffer();
    }
    
    public final void parse() throws IOException, DBException {
        int i;
        while ((i = this.reader.read()) != -1) {
            char c = (char)i;
            if (c == '\r' || c == '\n') {
                if (this.line.length() != 0) {
                    this.sql.append(this.line.toString());
                    this.sql.append("\r\n");
                    this.line.setLength(0);
                }
                this.isLineComment = false;
            }
            else {
                if (this.isLineComment) {
                    continue;
                }
                if (this.isBlockComment && this.isIgnoreComment) {
                    if (c != '*') {
                        continue;
                    }
                    i = this.reader.read();
                    c = (char)i;
                    if (c == -1 || c != '/') {
                        continue;
                    }
                    this.isBlockComment = false;
                }
                else {
                    switch (c) {
                        case '\'': {
                            this.line.append(c);
                            if (!this.isBlockComment) {
                                this.isString = !this.isString;
                                continue;
                            }
                            continue;
                        }
                        case '-': {
                            if (this.isBlockComment || this.isString) {
                                this.line.append('-');
                                continue;
                            }
                            i = this.reader.read();
                            c = (char)i;
                            if (c == '-') {
                                this.isLineComment = true;
                                continue;
                            }
                            this.line.append('-').append(c);
                            continue;
                        }
                        case '/': {
                            if (this.isBlockComment || this.isString) {
                                this.line.append('/');
                                continue;
                            }
                            i = this.reader.read();
                            c = (char)i;
                            if (i == -1) {
                                continue;
                            }
                            if (c != '*') {
                                this.line.append('/').append(c);
                                continue;
                            }
                            this.isBlockComment = true;
                            if (!this.isIgnoreComment) {
                                this.line.append('/').append(c);
                                continue;
                            }
                            continue;
                        }
                        case '*': {
                            if (!this.isBlockComment) {
                                this.line.append('*');
                                continue;
                            }
                            c = (char)this.reader.read();
                            if (c == -1) {
                                continue;
                            }
                            if (c == '/') {
                                this.isBlockComment = false;
                            }
                            if (!this.isIgnoreComment) {
                                this.line.append('*').append(c);
                                continue;
                            }
                            continue;
                        }
                        case ';': {
                            if (!this.isBlockComment && !this.isString) {
                                this.excute();
                                continue;
                            }
                            this.line.append(';');
                            continue;
                        }
                        case 'g': {
                            if (this.isBlockComment || this.isString) {
                                this.line.append('g');
                                continue;
                            }
                            i = this.reader.read();
                            c = (char)i;
                            if (c != 'O' && c != 'o') {
                                this.line.append('g').append(c);
                                continue;
                            }
                            i = this.reader.read();
                            final char nextChar = (char)i;
                            if (nextChar == '\r' || nextChar == '\n') {
                                this.excute();
                                continue;
                            }
                            this.line.append('g').append(c).append(nextChar);
                            continue;
                        }
                        case 'G': {
                            if (this.isBlockComment || this.isString) {
                                this.line.append('G');
                                continue;
                            }
                            i = this.reader.read();
                            c = (char)i;
                            if (c != 'O' && c != 'o') {
                                this.line.append('G').append(c);
                                continue;
                            }
                            i = this.reader.read();
                            final char nextChar = (char)i;
                            if (nextChar == '\r' || nextChar == '\n') {
                                this.excute();
                                continue;
                            }
                            this.line.append('G').append(c).append(nextChar);
                            continue;
                        }
                        default: {
                            this.line.append(c);
                            continue;
                        }
                    }
                }
            }
        }
        this.excute();
    }
    
    public boolean isIgnoreComment() {
        return this.isIgnoreComment;
    }
    
    private void excute() throws DBException {
        this.sql.append(this.line.toString());
        if (this.sql.length() != 0) {
            this.excuteSingleSql(this.sql.toString());
        }
        this.line.setLength(0);
        this.sql.setLength(0);
    }
    
    protected abstract void excuteSingleSql(final String p0) throws DBException;
}
